# coding:utf8
from . import real, minuteKline, kline,dcsreal


def use(source:str, minuteType:int=5, nodeCount:int=10, fqType:int=None, dateType:int=1):
    """
    :param source: 获取数据源类型：minuteKline=分钟线; kline=日线; real=实时行情及五档盘口
    :param minuteType: 分钟线周期，可获取分钟：5, 15, 30, 60
    :param nodeCount: 获取k线个数：一次最多600
    :param fqType:日K线复权类型：不填，则默认不复权；1，前复权；2，后复权
    :param dateType:日K线类型：不填则 默认 1,日线；2，周线；3，月线
    """
    if source in ["real"]:
        return real.Real()
    if source in ["dcsReal"]:
        return dcsreal.DcsReal()
    if source in ["minuteKline"]:
        return minuteKline.MinuteKline(minuteType=minuteType, nodeCount=nodeCount)
    if source in ["dayKline"]:
        return kline.Kline(fqType=fqType, dateType=dateType, nodeCount=nodeCount)
    raise NotImplementedError


# coding=utf-8
# !/usr/bin/python

"""
@Author: jeevan
@微信公众号: 大操手量化投资
@Site: http://dacaoshou.com
@Software: pycharm
@File：dcsreal.py
@Time：2021/1/12 19:33
@Description: 
"""
import json

import requests

from dcsmarket import basequotation
import pandas as pd

from dcsmarket.helpers import get_stock_info_prefix, get_token

__all__ = ['DcsReal']
class DcsReal(basequotation.BaseQuotation):
    """
    "stockCode"：股票编码
    "stockName"：股票名称
    "tradeDate"：交易日期
    "time"：实时时间
    "current"：现价
    "open"：开盘价
    "high"：最高价
    "low"：最低价
    "preClosePrice"：昨收盘价
    "upLimitPrice"：涨停价
    "downLimitPrice"：跌停价
    "uptickPrice"：涨跌
    "uptickRate"：涨跌幅
    "surgeRate"：振幅
    "vol"：成交量
    "amount"：成交总额
    "committeeSent"：委差
    "buyPriceList"：委买五档盘口挂单价
    "buyVol"：委买五档盘口挂单量
    "askPriceList"：委卖五档盘口挂单价
    "askVol"：委卖五档盘口挂单量
    """
    max_num = 100

    def __init__(self):
        self.headers = {
            "token": get_token()
        }
        self._session = requests.session()

    @property
    def stock_api(self) -> str:
        return f"/api/data/stock/real-time?stockCodeArray="

    def _gen_stock_prefix(self, stock_codes):
        result = []
        for code in stock_codes:
            dcs_code = get_stock_info_prefix(code)
            result.append(dcs_code)
        return result

    def format_response_data(self, rep_data, prefix=False):
        # 多个股票
        result = pd.DataFrame()
        columns = ["stockCode", "stockName", "tradeDate", "time", "current", "open",
                   "high", "low", "preClosePrice", "upLimitPrice", "downLimitPrice", "stockCode",
                   "uptickPrice", "uptickRate", "surgeRate", "vol", "amount", "committeeSent",
                   "buyPriceList", "buyVol", "askPriceList", "askVol"]
        for item in rep_data:
            data_dict = json.loads(item)
            if data_dict['code'] != 200:
                print(data_dict["message"])
                return
            data = data_dict['data']
        return data



# coding=utf-8
# !/usr/bin/python

"""
@Author: liao
@微信公众号: 大操手量化投资
@Site: http://dacaoshou.com
@Software: pycharm
@File：kline.py
@Time：2021/1/11 22:23
@Description: 
"""
import json

import pandas as pd
import requests

from dcsmarket import basequotation
from dcsmarket.helpers import get_stock_info
from dcsmarket.helpers import get_token


class Kline(basequotation.BaseQuotation):
    # 每次请求只能允许一个
    max_num = 1

    def __init__(self, fqType, dateType, nodeCount):
        self.headers = {
            "token": get_token()
        }
        self._session = requests.session()
        self.nodeCount = nodeCount
        self.fqType = "" if fqType is None else dateType
        self.dateType = 1 if dateType is None else dateType

    @property
    def stock_api(self) -> str:
        return "/api/data/stock/k-line-day?"

    def _gen_stock_prefix(self, stock_codes):
        result = []
        for code in stock_codes:
            code, marketCode, stockMarket = get_stock_info(code)
            param = "stockCode={}&nodeCount={}&marketCode={}&stockMarket={}&fqType={}&dateType={}".format(code,
                                                                                                          self.nodeCount,
                                                                                                          marketCode,
                                                                                                          stockMarket,
                                                                                                          self.fqType,
                                                                                                          self.dateType)
            result.append(param)
        return result

    def format_response_data(self, rep_data, **kwargs):
        # 多个股票
        stock_dict = {}
        columns = ["dateTime", "open", "high", "low", "close", "vol", "amount"]
        for item in rep_data:
            data_dict = json.loads(item)
            if data_dict['code'] != 200:
                print(data_dict["message"])
                return
            data = data_dict['data']
            p_data = pd.DataFrame(data["klineData"], columns=columns)
            p_data["stockCode"] =data["stockCode"]
            stock_dict.update({
                data["stockCode"]: p_data
            })
        return stock_dict



# coding:utf8
"""
@Author: liao
@微信公众号: 大操手量化投资
@Site: http://dacaoshou.com
@Software: pycharm
@Time：2021/4/2 15:14
@Description:
"""
import json
import os

import pandas as pd
import requests

STOCK_CODE_PATH = os.path.join("all_stock.csv")


def update_stock_codes():
    """获取所有股票 ID 到 all_stock_code 目录下"""
    try:
        headers = {
            "token": get_token()
        }
        _session = requests.session()
        response = _session.get("https://dacaoshou.com/api/data/stock/get_all_stock_info", headers=headers)
        data = json.loads(response.text)
        if data.get("code") != 0:
            print("获取股票基础数据失败，返回信息：{}，使用上次数据.".format(data.get("code")))
            return pd.DataFrame(STOCK_CODE_PATH)
        columns = ["dcsCode", "stockCode", "name", "area", "industry", "listedDate", "isHs"]
        p_data = pd.DataFrame(data.get("data"), columns=columns)
        p_data.to_csv(STOCK_CODE_PATH)
        return p_data
    except:
        print("获取股票基础数据失败，使用上次数据.")
    return pd.read_csv(STOCK_CODE_PATH)


def get_stock_codes(realtime=False):
    if realtime:
        return update_stock_codes()
    p_data = pd.DataFrame(STOCK_CODE_PATH)
    return p_data

def get_stock_type(stock_code):
    """判断股票ID对应的证券市场
    匹配规则
    ['50', '51', '60', '90', '110'] 为 sh
    ['00', '13', '18', '15', '16', '18', '20', '30', '39', '115'] 为 sz
    ['5', '6', '9'] 开头的为 sh， 其余为 sz
    :param stock_code:股票ID, 若以 'sz', 'sh' 开头直接返回对应类型，否则使用内置规则判断
    :return 'sh' or 'sz'"""
    assert type(stock_code) is str, "stock code need str type"
    sh_head = ("50", "51", "60", "90", "110", "113",
               "132", "204", "5", "6", "9", "7")
    if stock_code.startswith(("sh", "sz", "zz")):
        return stock_code[:2]
    else:
        return "sh" if stock_code.startswith(sh_head) else "sz"


def get_stock_info(stock_code):
    assert type(stock_code) is str, "请输入字符串类型的编码"
    sh_head = ("50", "51", "60", "90", "110", "113",
               "132", "204", "5", "6", "9", "7")
    code = stock_code
    if stock_code.startswith(("sh", "sz")):
        code = stock_code[2:]
    if code.startswith(sh_head):
        return code, "SH", 1
    else:
        return code, "SZ", 2


def get_stock_info_prefix(stock_code):
    assert type(stock_code) is str, "请输入字符串类型的编码"
    sh_head = ("50", "51", "60", "90", "110", "113",
               "132", "204", "5", "6", "9", "7")
    if stock_code.startswith(("sh", "sz")):
        return stock_code
    if stock_code.startswith(sh_head):
        return "sh" + stock_code
    else:
        return "sz" + stock_code


def set_token(token):
    df = pd.DataFrame([token.strip()], columns=['token'])
    user_home = os.path.expanduser('~')
    fp = os.path.join(user_home, 'dcs_token.csv')
    df.to_csv(fp, index=False)


def get_token():
    user_home = os.path.expanduser('~')
    fp = os.path.join(user_home, 'dcs_token.csv')
    if os.path.exists(fp):
        df = pd.read_csv(fp)
        return str(df.loc[0]['token'])
    else:
        print("token校验失败，请使用token获取数据，token获取请访问：https://dacaoshou.com")
        return None


def load_token():
    user_home = os.path.expanduser('~')
    fp = os.path.join(user_home, 'dcs_token.csv')
    if os.path.exists(fp):
        df = pd.read_csv(fp)
        return str(df.loc[0]['token'])
    else:
        return None



# coding:utf8
"""
@Author: cat
@微信公众号: 大操手量化投资
@Site: http://dacaoshou.com
@Software: pycharm
@Time：2021/4/2 15:14
@Description:
"""
import abc
import json
import multiprocessing.pool
import warnings

import requests

from dcsmarket.helpers import get_token
from . import helpers

class BaseQuotation(metaclass=abc.ABCMeta):
    """行情获取基类"""
    max_num = 800  # 每次请求的最大股票数

    @property
    def dcs_domain(self) -> str:
        return "https://dacaoshou.com"

    @property
    @abc.abstractmethod
    def stock_api(self) -> str:
        pass

    def __init__(self):
        self.headers = {
            "token": get_token()
        }
        self._session = requests.session()
        stock_codes = self.load_stock_codes()
        self._can_multi = True
        self.stock_list = self.gen_stock_list(stock_codes)

    def gen_stock_list(self, stock_codes):
        stock_with_exchange_list = self._gen_stock_prefix(stock_codes)

        if self.max_num > len(stock_with_exchange_list):
            request_list = ",".join(stock_with_exchange_list)
            return [request_list]

        stock_list = []
        for i in range(0, len(stock_codes), self.max_num):
            request_list = ",".join(
                stock_with_exchange_list[i : i + self.max_num]
            )
            stock_list.append(request_list)
        return stock_list

    def _gen_stock_prefix(self, stock_codes):
        return [
            helpers.get_stock_type(code) + code[-6:] for code in stock_codes
        ]

    @staticmethod
    def load_stock_codes():
        with open(helpers.STOCK_CODE_PATH) as f:
            return json.load(f)["stock"]

    @property
    def all(self):
        warnings.warn("use market_snapshot instead", DeprecationWarning)
        return self.get_stock_data(self.stock_list)

    @property
    def all_market(self):
        return self.get_stock_data(self.stock_list, prefix=True)

    def stocks(self, stock_codes, prefix=False):
        """
        :param stock_codes: 股票数组，例如：["601988","600018"]
        :return:
        """
        return self.real(stock_codes, prefix)

    def real(self, stock_codes, prefix=False):
        if not isinstance(stock_codes, list):
            stock_codes = [stock_codes]

        stock_list = self.gen_stock_list(stock_codes)
        return self.get_stock_data(stock_list, prefix=prefix)

    def market_snapshot(self, prefix=False):
        return self.get_stock_data(self.stock_list, prefix=prefix)

    def get_stocks_by_range(self, params):
        try:
            headers = {
                "Accept-Encoding": "gzip, deflate, sdch"
            }
            headers.update(self.headers)
            if self.stock_api.find("http://")>=0 or self.stock_api.find("https://")>=0:
                _stock_api = self.stock_api
            else:
                _stock_api = self.dcs_domain+self.stock_api
            r = self._session.get(_stock_api + params, headers=headers)
            return r.text
        except:
            print("获取数据失败，参数：{}".format(params))
            return None

    def get_stock_data(self, stock_list, **kwargs):
        """获取并格式化股票信息"""
        res = self._fetch_stock_data(stock_list)
        return self.format_response_data(res, **kwargs)

    def _fetch_stock_data(self, stock_list):
        """获取股票信息"""
        pool = multiprocessing.pool.ThreadPool(len(stock_list))
        try:
            res = pool.map(self.get_stocks_by_range, stock_list)
        finally:
            pool.close()
        return [d for d in res if d is not None]

    def format_response_data(self, rep_data, **kwargs):
        pass



# coding:utf8
import re
import time

from . import basequotation


class Real(basequotation.BaseQuotation):
    """
    新浪接口：实时行情获取
    每次获取最大数:800只个股，频率不得超过
    """

    max_num = 800

    def __init__(self):
        pass

    grep_detail = re.compile(
        r"(\d+)=[^\s]([^\s,]+?)%s%s"
        % (r",([\.\d]+)" * 29, r",([-\.\d:]+)" * 2)
    )
    grep_detail_with_prefix = re.compile(
        r"(\w{2}\d+)=[^\s]([^\s,]+?)%s%s"
        % (r",([\.\d]+)" * 29, r",([-\.\d:]+)" * 2)
    )
    del_null_data_stock = re.compile(
        r"(\w{2}\d+)=\"\";"
    )

    @property
    def stock_api(self) -> str:
        return f"http://hq.sinajs.cn/rn={int(time.time() * 1000)}&list="

    def format_response_data(self, rep_data, prefix=False):
        stocks_detail = "".join(rep_data)
        stocks_detail = self.del_null_data_stock.sub('', stocks_detail)
        stocks_detail = stocks_detail.replace(' ', '')
        grep_str = self.grep_detail_with_prefix if prefix else self.grep_detail
        result = grep_str.finditer(stocks_detail)
        stock_dict = dict()
        for stock_match_object in result:
            stock = stock_match_object.groups()
            stock_dict[stock[0]] = dict(
                name=stock[1],
                open=float(stock[2]),
                close=float(stock[3]),
                now=float(stock[4]),
                high=float(stock[5]),
                low=float(stock[6]),
                buy=float(stock[7]),
                sell=float(stock[8]),
                turnover=int(stock[9]),
                volume=float(stock[10]),
                bid1_volume=int(stock[11]),
                bid1=float(stock[12]),
                bid2_volume=int(stock[13]),
                bid2=float(stock[14]),
                bid3_volume=int(stock[15]),
                bid3=float(stock[16]),
                bid4_volume=int(stock[17]),
                bid4=float(stock[18]),
                bid5_volume=int(stock[19]),
                bid5=float(stock[20]),
                ask1_volume=int(stock[21]),
                ask1=float(stock[22]),
                ask2_volume=int(stock[23]),
                ask2=float(stock[24]),
                ask3_volume=int(stock[25]),
                ask3=float(stock[26]),
                ask4_volume=int(stock[27]),
                ask4=float(stock[28]),
                ask5_volume=int(stock[29]),
                ask5=float(stock[30]),
                date=stock[31],
                time=stock[32],
            )
        return stock_dict



# coding=utf-8
# !/usr/bin/python

"""
@Author: jeevan
@微信公众号: 大操手量化投资
@Site: http://dacaoshou.com
@Software: pycharm
@File：setup.py.py
@Time：2021/4/2 15:14
@Description: 
"""
from distutils.core import setup
from setuptools import find_packages

setup(name='dcsmarket',  # 包名
      version='1.8.3.1',  # 版本号
      description='大操手量化投资，股票行情数据接口',
      long_description='大操手量化投资，股票行情数据接口',
      author='大操手量化投资',
      author_email='yecaimeiv1@163.com',
      url='https://dacaoshou.com',
      license='',
      install_requires=['pandas'],
      classifiers=[
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Natural Language :: Chinese (Simplified)',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Topic :: Utilities'
      ],
      keywords=['stock','data','大操手'],
      packages=['dcsmarket'],  # 必填，就是包的代码主目录
      include_package_data=True,
      )
# !/usr/bin/env python
